#!/usr/bin/python3
__copyright__ = """
 SPDX-FileCopyrightText: Copyright (c) 2024-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 SPDX-License-Identifier: BSD-3-Clause

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

# Module that contain classes DPADBInit and SimDataManager

from ..simenums.simenums import DPADB
from ..simyaml.simyaml import YAMLSimOperations


# Class for get keys of external (file) database and to right init part of it
class DPADBInit:
    def __init__(self):
        self.dpa_dict = {DPADB.CHIP: {},
                         DPADB.VHCA: [],
                         DPADB.PARTITIONS: [],
                         DPADB.ROOT_GROUPS: [],
                         DPADB.PROCESSES: {}}

    def keys(self):
        return self.dpa_dict.keys()

    def init(self, key):
        return self.dpa_dict[key]


# Class for read data from the YAML file to external database
class SimDataManager:
    def __init__(self):
        self._data = {}
        for key in DPADBInit().keys():
            self._data[key] = DPADBInit().init(key)

    def read_data(self, filename):
        self._data = YAMLSimOperations().read_database_file(filename, self._data)
        return self._data

    @property
    def data(self):
        return self._data
