#!/usr/bin/python3
__copyright__ = """
 SPDX-FileCopyrightText: Copyright (c) 2024-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 SPDX-License-Identifier: BSD-3-Clause

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

# Module that contain class DPAObjectsHandler

from ...simenums.simenums import DPADB
from ..operation import OperationHandler


# Class DPAObjectsHandler derived from OperationHandler is a base class
class DPAObjectsHandler(OperationHandler):
    def __init__(self, arg_params, filename):
        super().__init__(arg_params, filename)
        self._partition_list = self._data[DPADB.PARTITIONS]

    def _query_elements(self, elem_list):
        index = 0
        if elem_list and len(elem_list) > 0:
            for elem in elem_list:
                if self._check_for_query(elem):
                    index += 1
                    print(f"{index}) ", end="")
                    self._print_single_elem(elem)
                    print("")
        print(f"In total there are {index} EU {self._name}s configured.")

    def _set_update(self):
        self._data[DPADB.PARTITIONS] = self._partition_list
        super()._set_update()

    @property
    def _name(self):
        return ""

    def _print_single_elem(self, _):
        pass

    def _check_for_query(self, _):
        return True

    def _find_free_id(self, list_to_find):
        if not list_to_find or len(list_to_find) == 0:
            return 1
        existed_list = []
        for elem in list_to_find:
            existed_list.append(elem['id'])
        for ident in range(1, 1024):
            if ident not in existed_list:
                return ident
        return ident
