#!/usr/bin/python3
__copyright__ = """
 SPDX-FileCopyrightText: Copyright (c) 2024-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 SPDX-License-Identifier: BSD-3-Clause

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

# Module that contain class InfoHandler

from ..operation import OperationHandler
from ...simenums.simenums import DPADB
from ...simyaml.simyaml import YAMLSimOperations


# Class InfoHandler derived from base class OperationHandler
# for print information about chip parameters, VHCA and current
# status of the chip
class InfoHandler(OperationHandler):
    def _get_subfunction(self):
        subfunc_dict = {"chip": self.__chip_info,
                        "vhca": self.__vhca_info,
                        "status": self.__status_info}
        return subfunc_dict[self._params.subcommand]

    def __chip_info(self):
        YAMLSimOperations().print_chip_dict(self._data[DPADB.CHIP], self._params.yaml_file)

    def __vhca_info(self):
        YAMLSimOperations().print_vhca_dict(self._data[DPADB.VHCA], self._params.yaml_file)

    def __status_info(self):
        status_dict = {}
        status_dict['number_of_partitions'] = len(self._data[DPADB.PARTITIONS]) \
            if self._data[DPADB.PARTITIONS] else 0
        status_dict['number_of_root_groups'] = len(self._data[DPADB.ROOT_GROUPS]) \
            if self._data[DPADB.ROOT_GROUPS] else 0
        status_dict['number_of_processes'] = self._data[DPADB.PROCESSES]['processes'] \
            if self._data[DPADB.PROCESSES] else 0
        YAMLSimOperations().print_dict(status_dict, self._params.yaml_file)
